/**********************************************************/
/**                                                       */
/**   %Z% %M% Version %I% (%G%)                           */
/**                                                       */
/**********************************************************/
/**                                                       */
/**   AUTEUR :  O.DEBON                                   */
/**                                                       */
/**   DATE DE CREATION :  23/12/96                        */
/**                                                       */
/**********************************************************/
/**                                                       */
/**   MODIFICATIONS :                                     */
/**                                                       */
/**   Date         Auteur           Commentaires          */
/**                                                       */
/**                                                       */
/**********************************************************/
static char *sccs_id = "%Z% %M% Version %I% (%G%)";
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "puck.h"

extern Puck puck;
extern Game game;
extern Racket racket2,racket;

/* PRIVATE */
static int auto_num = 0;

/* Declarations */

static void init_racket_default();
static void move_racket_default();
static void init_racket_rough();
static void move_racket_rough();

/* Registrations */
struct s_auto {
	void (*init)();
	void (*move)();
} AutoPlayer[] = {
	{ init_racket_default, move_racket_default } ,
	{ init_racket_rough, move_racket_rough } ,
	{ (void(*)()) 0, (void(*)()) 0 }
};

void
init_automatic()
{
	game.init_racket2 = AutoPlayer[auto_num].init;
	game.move_racket2 = AutoPlayer[auto_num].move;
	auto_num++;
	if (AutoPlayer[auto_num].init == 0 || AutoPlayer[auto_num].move == 0)
	{
		auto_num = 0;
	}
	game.init_racket2();
}

/*************** DEFAULT ************/
/*** Behaviour :
	It follows the puck, but it is limited in lateral speed.
	It normally it just opposes the racket.
	If Player if far (a quarter of Field width) from puck, it
	strikes the puck hard.
 ****/

void
init_racket_default()
{
	racket2.x = GAME_DEPTH-GAME_PLAYER_ZONE/2;
	racket2.y = 0.0;
	racket2.last_x = racket2.x;
	racket2.last_y = racket2.y;
}

/* Automatic Game Mode */
void
move_racket_default()
{
#define YMAX PUCK_WIDTH/3
	racket2.last_x = racket2.x;
	racket2.last_y = racket2.y;
	racket2.y = puck.y*(puck.x/GAME_DEPTH);
	racket2.x = 2*GAME_DEPTH-puck.x-GAME_PLAYER_ZONE;
	if (racket2.x > GAME_DEPTH)
	{
		racket2.x = GAME_DEPTH;
	}
	if (racket2.y-racket2.last_y > YMAX && puck.x-puck.last_x > YMAX)
	{
		racket2.y = racket2.last_y+YMAX;
	}
	if (racket2.y-racket2.last_y < -YMAX && puck.x-puck.last_x > YMAX)
	{
		racket2.y = racket2.last_y-YMAX;
	}
	if (racket2.y < (-GAME_WIDTH/2+RACKET_WIDTH/2)) racket2.y = -GAME_WIDTH/2+RACKET_WIDTH/2;
	if (racket2.y > (GAME_WIDTH/2-RACKET_WIDTH/2)) racket2.y =  GAME_WIDTH/2-RACKET_WIDTH/2;
	if (abs((int)(racket.y-puck.y)) > GAME_WIDTH/4)
	{
		if (puck.x >= GAME_DEPTH-GAME_PLAYER_ZONE)
		{
			racket2.dx = -PUCK_DXY_MAX;
			racket2.dy = -puck.dy;
			check_colision_racket_with_puck();
		}
	}
	else
	{
		racket2.dx = 0;
		racket2.dy = 0;
	}
#undef YMAX
}

/*************** ROUGH ************/
/*** Behaviour :
	It follows the puck, but it is limited in lateral speed.
	It strikes the puck hard.
 ****/

void
init_racket_rough()
{
	racket2.x = GAME_DEPTH-GAME_PLAYER_ZONE/2;
	racket2.y = 0.0;
	racket2.last_x = racket2.x;
	racket2.last_y = racket2.y;
	strcpy(game.opponent_name,"Bull");
}

/* Automatic Game Mode */
void
move_racket_rough()
{
#define YMAX PUCK_WIDTH/4
	racket2.last_x = racket2.x;
	racket2.last_y = racket2.y;
	racket2.y = puck.y*(puck.x/GAME_DEPTH);
	racket2.x = 2*GAME_DEPTH-puck.x-GAME_PLAYER_ZONE;
	if (racket2.x > GAME_DEPTH)
	{
		racket2.x = GAME_DEPTH;
	}
	if (racket2.y-racket2.last_y > YMAX && puck.x-puck.last_x > YMAX)
	{
		racket2.y = racket2.last_y+YMAX;
	}
	if (racket2.y-racket2.last_y < -YMAX && puck.x-puck.last_x > YMAX)
	{
		racket2.y = racket2.last_y-YMAX;
	}
	if (racket2.y < (-GAME_WIDTH/2+RACKET_WIDTH/2)) racket2.y = -GAME_WIDTH/2+RACKET_WIDTH/2;
	if (racket2.y > (GAME_WIDTH/2-RACKET_WIDTH/2)) racket2.y =  GAME_WIDTH/2-RACKET_WIDTH/2;
	if (puck.x >= GAME_DEPTH-GAME_PLAYER_ZONE)
	{
		racket2.dx = -PUCK_DXY_MAX;
		racket2.dy = -puck.dy/2;
		check_colision_racket_with_puck();
	}
#undef YMAX
}
