/**********************************************************/
/**                                                       */
/**   %Z% %M% Version %I% (%G%)                           */
/**                                                       */
/**********************************************************/
/**                                                       */
/**   AUTEUR :  O.DEBON                                   */
/**                                                       */
/**   DATE DE CREATION :  23/12/96                        */
/**                                                       */
/**********************************************************/
/**                                                       */
/**   MODIFICATIONS :                                     */
/**                                                       */
/**   Date         Auteur           Commentaires          */
/**                                                       */
/**                                                       */
/**********************************************************/
static char *sccs_id = "%Z% %M% Version %I% (%G%)";
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "puck.h"

/* GLOBALS */
/* X11 variables */
GC              gc;
Display        *dpy;
long            white, black;
Window          frame, field, panel;
long            depth;
XFontStruct    *font[NB_FONTS];

/* Game variable */
Game		game;

/* PRIVATE */
static void delete_cursor();
static void main_loop_slave();
static void main_loop_master_auto();
static void display_ready_flags();
static void display_game_over();
static void display_replay();
static void display_intro();
static void set_wm_hints();
static void delete_cursor();

/* EXTERNALS */
extern void init_automatic();

/* MAIN */
main(int argc, char **argv)
{
	char           *display_var;
	char	       *appname;

	appname = argv[0];

	display_var = (char *) getenv("DISPLAY");

	if (display_var == 0)
		display_var = ":0";

	dpy = XOpenDisplay(display_var);
	if (dpy == 0)
	{
		printf("Can't open display %s\n",display_var);
		exit(1);
	}

	game.xpm_file[0] = 0;
	game.mode = GAME_MASTER;
	game.port = GAME_PORT;
	strncpy(game.my_name , (char *)cuserid(0), MAX_NAME-1);
	game.my_name[MAX_NAME-1] = 0;
	gethostname(game.hostname,MAX_NAME-1);

	/* Extract options */
	for(argc--,argv++; argc; argc--,argv++)
	{
		if (!strcmp(*argv,"-host"))
		{
			if (game.mode == GAME_AUTO)
			{
				printf("Ambiguous options: -host and -noclient\n");
				exit(1);
			}
			else
			{
				argc--; argv++;
				if (*argv == 0)
				{
					printf("No argument for -host\n");
					exit(1);
				}
				game.server_name = *argv;
				game.mode = GAME_SLAVE;
			}
		}
		if (!strcmp(*argv,"-port"))
		{
			argc--; argv++;
			if (*argv == 0)
			{
				printf("No argument for -port\n");
				exit(1);
			}
			game.port = atoi(*argv);
		}
		if (!strcmp(*argv,"-xpm"))
		{
			argc--; argv++;
			if (*argv == 0)
			{
				printf("No argument for -xpm\n");
				exit(1);
			}
			strncpy(game.xpm_file, *argv, MAX_PATH-1);
			game.xpm_file[MAX_PATH-1] = 0;
		}
		if (!strcmp(*argv,"-name"))
		{
			argc--; argv++;
			if (*argv == 0)
			{
				printf("No argument for -name\n");
				exit(1);
			}
			strncpy(game.my_name , *argv, MAX_NAME-1);
			game.my_name[MAX_NAME-1] = 0;
		}
		if (!strcmp(*argv,"-noclient"))
		{
			if (game.mode == GAME_SLAVE)
			{
				printf("Ambiguous options: -host and -noclient\n");
				exit(1);
			}
			else
			{
				game.mode = GAME_AUTO;
			}
		}
		if (!strcmp(*argv,"-help"))
		{
			printf("Usage : %s options\n",appname);
			printf("   -help : this\n");
			printf("   -name <string> : player name.\n");
			printf("   -host <name> : name of remote server.\n");
			printf("   -noclient : game is standalone.\n");
			printf("   -port : server udp port number. Optional\n");
			printf("   -xpm : XPM file picture of you for your opponent. Absolute path name needed. Optional\n");
			printf("Xpuck V%s, 1997, <Olivier Debon> odebon@club-internet.fr\n",GAME_VERSION);
			exit(0);
		}
	}

	switch (game.mode)
	{
		case GAME_MASTER:
			game.server = init_server(game.port);
			break;
		case GAME_SLAVE:
			init_client(game.server_name,game.port);
			game.server = init_server(game.port+1);
			connect_to_server();
			break;
		case GAME_AUTO:
			init_automatic();
			game.server = 0;
			strcpy(game.opponent_name,"Puck Man");
			break;
	}

	/* Failed in the two cases above */
	if (game.server < 0)
	{
		printf("Failed to connect.");
		game.mode = GAME_AUTO;
	}

	if (game.mode == GAME_MASTER)
	{
		printf("Waiting for a remote player ...\n");
		game.client = wait_for_client();
	}

	/* XSynchronize(dpy,True); */
	gc = DefaultGC(dpy, DefaultScreen(dpy));
	white = WhitePixel(dpy, DefaultScreen(dpy));
	black = BlackPixel(dpy, DefaultScreen(dpy));
	depth = XDefaultDepth(dpy, DefaultScreen(dpy));
	font[FONT_TITLE] = XLoadQueryFont(dpy, "-*-courier-bold-r-*--24-*");
	font[FONT_TEXT] = XLoadQueryFont(dpy, "-*-courier-bold-r-*--12-*");
	font[FONT_DIGIT] = XLoadQueryFont(dpy, "-*-helvetica-bold-r-*--18-*");
	font[FONT_SCORE] = XLoadQueryFont(dpy, "-*-lucida-bold-r-normal-*-12-*");
	font[FONT_INTRO] = XLoadQueryFont(dpy, "-*-lucida-*-r-*-*-60-*-*-*-*-*-*-*");
	if (font[FONT_INTRO] == 0)
	{
		font[FONT_INTRO] = XLoadQueryFont(dpy,"12x24");
	}

	frame = XCreateSimpleWindow(dpy, RootWindow(dpy, DefaultScreen(dpy)),
				    0, 0,
				    FIELD_WIDTH + 2 * BORDER_WIDTH,
				    FIELD_HEIGHT + PANEL_HEIGHT + 2 * BORDER_WIDTH,
				    0, black, white);

	set_wm_hints();

	XSelectInput(dpy, frame, ExposureMask);
	XMapWindow(dpy, frame);

	panel = XCreateSimpleWindow(dpy, frame,
				    0, 0,
				    FIELD_WIDTH,
				    PANEL_HEIGHT,
				    BORDER_WIDTH, black, black);
	XMapWindow(dpy, panel);

	XSelectInput(dpy, panel, ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask);

	field = XCreateSimpleWindow(dpy, frame,
				    0, PANEL_HEIGHT,
				    FIELD_WIDTH,
				    FIELD_HEIGHT,
				    BORDER_WIDTH, black, black);
	XMapWindow(dpy, field);

	XSelectInput(dpy, field,
			ExposureMask |
			ButtonPressMask |
			PointerMotionMask |
			KeyPressMask);

	/* Make an invisible cursor within field */
	delete_cursor();

	/* Pixmap creations and color initialization */
	create_pixmaps();

	XFlush(dpy);

	/* Game loop */
	while (1)
	{
		switch (game.mode)
		{
			case GAME_AUTO:
			case GAME_MASTER:
				main_loop_master_auto();
				break;
			case GAME_SLAVE:
				main_loop_slave();
		}
	}
}

static void
set_wm_hints()
{
	XWMHints        wmhints;
	XSizeHints      sizehints;
	Atom            wm_delete_win;  /* WM Quit */

	/* Window Manager Informations */
	switch (game.mode)
	{
		case GAME_MASTER:
			XStoreName(dpy, frame, "Xpuck - Master");
			XSetIconName(dpy, frame, "Xpuck - Master");
			break;
		case GAME_SLAVE:
			XStoreName(dpy, frame, "Xpuck - Slave");
			XSetIconName(dpy, frame, "Xpuck - Slave");
			break;
		case GAME_AUTO:
			XStoreName(dpy, frame, "Xpuck");
			XSetIconName(dpy, frame, "Xpuck");
			break;
	}
	/* Not a resizable window */
        sizehints.min_width = FIELD_WIDTH + 2 * BORDER_WIDTH;
        sizehints.min_height = FIELD_HEIGHT + PANEL_HEIGHT + 2 * BORDER_WIDTH;
        sizehints.max_width = FIELD_WIDTH + 2 * BORDER_WIDTH;
        sizehints.max_height = FIELD_HEIGHT + PANEL_HEIGHT + 2 * BORDER_WIDTH;
        sizehints.flags = PMinSize | PMaxSize;
        XSetNormalHints(dpy, frame, &sizehints);
	/* Catch Quit or Delete or Destroy from WM */
        wm_delete_win = XInternAtom(dpy, "WM_DELETE_WINDOW", True);
        XSetWMProtocols(dpy, frame, &wm_delete_win, 1);
}

static void
delete_cursor()
{
	Cursor	cursor;
	Font	font;
	XColor	color;

	color.flags = 0;
	color.red = 0;
	color.green = 0;
	color.blue = 0;
	font = XLoadFont(dpy,"9x15");
	cursor = XCreateGlyphCursor(dpy,font,font,' ',' ',&color,&color);
	XUnloadFont(dpy,font);
	XDefineCursor(dpy,field,cursor);
}

/* Events manager */
static void
main_loop_master_auto()
{
	XEvent          event;
	fd_set		fdset;
	struct timeval  timeout,start,stop;
	int		status = 0;
	long		elapsed;


	game.state = S_INTRO;

	XClearWindow(dpy, panel);

	XFlush(dpy);

	reset_score();
	game.to_serve = 0;

	while (1)
	{
		if (game.state != S_PLAY && game.state != S_START)
		{
			XUngrabPointer(dpy,CurrentTime);
		}
		switch (game.state)
		{
		case S_LOST:
			game.state = S_NEW;
			if (game.to_serve)
			{
				game.to_serve = 0;
			}
			else
			{
				game.to_serve = 1;
			}
			break;
		case S_NEW:
			/* New Stage */
			XClearWindow(dpy,field);
			XClearWindow(dpy,panel);
			draw_field();
			clear();
			game.ready[0] = 0;
			if (game.mode == GAME_AUTO)
			{
				game.ready[1] = 1;
			}
			else
			{
				game.ready[1] = 0;
			}
			if (game.mode == GAME_MASTER)
			{
				send_slave();
			}
			game.state = S_START;
			redraw();
			if (match_is_over() >= 0)
			{
				reset_score();
				if (game.mode == GAME_AUTO)
				{
					init_automatic();
				}
			}
			if (score(0,0))
			{
				game.set++;
			}
			display_score();
			if (game.mode == GAME_MASTER)
			{
				send_slave();
			}
			display_ready_flags();
			break;

		case S_GAME_OVER:
			display_game_over();
			break;

		case S_REPLAY:
			FD_ZERO(&fdset);
			FD_SET(ConnectionNumber(dpy),&fdset);
			timeout.tv_sec = 0;
			timeout.tv_usec = 10*TICK;
			status = select(16,&fdset,0,0,&timeout);

			if (status == 0)
			{
				replay(1);
				redraw();
				display_replay();
				if (game.mode == GAME_MASTER)
				{
					send_slave();
				}
			}
			if (!FD_ISSET(ConnectionNumber(dpy),&fdset))
			{
				/* Skip the event reading */
				continue;
			}

			break;

		case S_START:
			FD_ZERO(&fdset);
			FD_SET(ConnectionNumber(dpy),&fdset);
			if (game.mode == GAME_MASTER)
			{
				FD_SET(game.client,&fdset);
			}
			status = select(16,&fdset,0,0,0);
			if (game.mode == GAME_MASTER && FD_ISSET(game.client,&fdset))
			{
				recv_slave();
				redraw();
				if (game.ready[0] && game.ready[1])
				{
					game.state = S_PLAY;
					init_puck();
				}
				send_slave();
				display_ready_flags();
			}
			if (!FD_ISSET(ConnectionNumber(dpy),&fdset))
			{
				/* Skip the event reading */
				continue;
			}
			break;

		case S_PLAY:
			/* Playing ... */
			FD_ZERO(&fdset);
			FD_SET(ConnectionNumber(dpy),&fdset);
			if (game.mode == GAME_MASTER)
			{
				FD_SET(game.client,&fdset);
			}

			timeout.tv_sec = 0;
			timeout.tv_usec = TICK;
			status = select(16,&fdset,0,0,&timeout);

			if (status == 0)
			{
				switch (game.mode)
				{
					case GAME_AUTO:
						game.move_racket2();
						redraw();
						advance_puck();
						break;
					case GAME_MASTER:
						advance_puck();
						send_slave();
						break;
				}
				move_racket(event);
				continue;
			}
			else
			{
				if (game.mode == GAME_MASTER && FD_ISSET(game.client,&fdset))
				{
					recv_slave();
					check_colision_racket_with_puck();
					redraw();
				}
				if (!FD_ISSET(ConnectionNumber(dpy),&fdset))
				{
					/* Skip the event reading */
					continue;
				}
			}
			break;
		} /* End switch */

		XNextEvent(dpy, &event);

		/* printf("Type : %d\n",event.type); */

		/* ClientMessage occurs on QUIT menu selection */
		if (event.type == ClientMessage)
		{
			if (game.mode== GAME_MASTER)
			{
				close_server();
			}
			XCloseDisplay(dpy);
			exit(0);
		}
		if (event.type == Expose)
		{
			XClearWindow(dpy,field);
			switch(game.state)
			{
			case S_INTRO:
				display_intro();
				break;
			case S_GAME_OVER:
				display_game_over();
				break;
			default:
				redraw();
				display_score();
				display_ready_flags();
				break;
			}
		}
		if (event.xany.window == field)
		{
			if ((event.type == ButtonPress)
			    && (event.xbutton.button == Button1))
			{
				switch (game.state)
				{
					case S_GAME_OVER:
					case S_INTRO:
						game.state = S_NEW;
						init_racket();
						break;
					case S_START:
						XGrabPointer(dpy,field,True,
								ButtonPressMask | PointerMotionMask,
								GrabModeAsync, GrabModeAsync,
								field, None, CurrentTime);
						game.ready[0] = 1;
						display_ready_flags();
						if (game.ready[0] && game.ready[1])
						{
							if (game.mode == GAME_MASTER)
							{
								send_slave();
							}
							game.state = S_PLAY;
							init_puck();
						}
						break;
				}
				if (game.mode == GAME_MASTER)
				{
					send_slave();
				}
			}
			if (event.type == MotionNotify)
			{
				if (game.state == S_START)
				{
					move_racket(event);
					if (game.mode == GAME_MASTER)
					{
						send_slave();
					}
					redraw();
				}
			}
			if (event.type == KeyPress)
			{
				XComposeStatus compose;
				KeySym keysym;

				XLookupString(&event.xkey,0,0,&keysym,&compose);
				switch (game.state)
				{
					case S_PLAY:
						if (keysym == XK_a)
						{
							game.state = S_GAME_OVER;
							display_game_over();
						}
						break;
					case S_START:
						if (keysym == XK_r)
						{
							replay(0);
							game.state = S_REPLAY;
						}
						if (keysym == XK_c)
						{
							init_automatic();
							game.state = S_NEW;
						}
						if (keysym == XK_q)
						{
							game.state = S_QUIT;
						}
						break;
					case S_REPLAY:
						game.state = S_START;
						break;
				}
				if (game.mode == GAME_MASTER)
				{
					send_slave();
				}
				if (game.state == S_QUIT)
				{
					exit(0);
				}
			}
		}
	} /* End while */
}

static void
main_loop_slave()
{
	XEvent          event;
	fd_set		fdset;
	struct timeval  timeout,start,stop;
	int		status = 0;
	long		elapsed;


	game.state = S_INTRO;

	XClearWindow(dpy, panel);
	draw_field();

	XFlush(dpy);

	reset_score();

	while (1)
	{
		/* Playing ... */
		FD_ZERO(&fdset);
		FD_SET(ConnectionNumber(dpy),&fdset);
		FD_SET(game.server,&fdset);
		status = select(16,&fdset,0,0,0);

		if (FD_ISSET(game.server,&fdset))
		{
			recv_master();
			redraw();
			if (game.state == S_REPLAY)
			{
				display_replay();
			}
			if (game.state == S_PLAY)
			{
				move_racket(event);
				send_master();
			}
		}

		if (game.state != S_PLAY && game.state != S_START)
		{
			XUngrabPointer(dpy,CurrentTime);
		}
		switch (game.state)
		{
		case S_NEW:
			/* New Stage */
			XClearWindow(dpy,field);
			XClearWindow(dpy,panel);
			clear();
			game.ready[0] = 0;
			if (game.score[0][0] == 0 && game.score[0][1] == 0 && game.set == 0)
			{
				reset_score();
			}
			display_score();
			display_ready_flags();
			break;
		case S_START:
			display_ready_flags();
			break;
		case S_GAME_OVER:
			redraw();
			display_score();
			display_game_over();
			break;
		case S_INTRO:
			display_intro();
			break;
		case S_QUIT:
			printf("Over.\n");
			exit(0);
			break;
		default:
			redraw();
			display_score();
			display_ready_flags();
			break;
		} /* End switch */

		if (!FD_ISSET(ConnectionNumber(dpy),&fdset))
		{
			/* Skip the event reading */
			continue;
		}

		XNextEvent(dpy, &event);

		/* printf("Type : %d\n",event.type); */

		/* ClientMessage occurs on QUIT menu selection */
		if (event.type == ClientMessage)
		{
			close_client();
			XCloseDisplay(dpy);
			exit(0);
		}
		if (event.type == Expose)
		{
			XClearWindow(dpy,field);
			switch(game.state)
			{
			case S_GAME_OVER:
				display_game_over();
				break;
			default:
				redraw();
				display_score();
				display_ready_flags();
				break;
			}
		}
		if (event.xany.window == field)
		{
			if ((event.type == ButtonPress)
			    && (event.xbutton.button == Button1))
			{
				if (game.state == S_START)
				{
					XGrabPointer(dpy,field,True,
							ButtonPressMask | PointerMotionMask,
							GrabModeAsync, GrabModeAsync,
							field, None, CurrentTime);
					game.ready[0] = 1;
					display_ready_flags();
					send_master();
				}
			}
			if (event.type == KeyPress)
			{
				XComposeStatus compose;
				KeySym keysym;

				XLookupString(&event.xkey,0,0,&keysym,&compose);
				switch (game.state)
				{
					case S_START:
						if (keysym == XK_q)
						{
							game.state = S_QUIT;
						}
						break;
				}
			}
			if (event.type == MotionNotify)
			{
				if (game.state == S_START)
				{
					move_racket(event);
					send_master();
					recv_master();
					redraw();
				}
			}
		}
	} /* End while */
}

static void
display_game_over()
{
	char msg[100];
	char *p;
	int width;

	XSetFont(dpy, gc, font[FONT_TITLE]->fid);
	switch (match_is_over())
	{
		case 0:
			/* Player is the winner */
			p = game.my_name;
			break;
		case 1:
			/* Opponent is the winner */
			p = game.opponent_name;
			break;
		case -1:
			/* Next Set */
			p = 0;
			strcpy(msg,"Get Ready For Next Set!");
			break;
	}
	if (p)
	{
		sprintf(msg,"%s is The Winner !",p);
		width = XTextWidth(font[FONT_TITLE],msg, strlen(msg));
		XSetForeground(dpy, gc, black);
		XDrawString(dpy, field, gc, FIELD_WIDTH/2-width/2+2, FIELD_HEIGHT/2+50+2, msg, strlen(msg));
		XSetForeground(dpy, gc, white);
		XDrawString(dpy, field, gc, FIELD_WIDTH/2-width/2, FIELD_HEIGHT/2+50, msg, strlen(msg));
		strcpy(msg,"Game Over");
	}
	width = XTextWidth(font[FONT_TITLE],msg, strlen(msg));
	XSetForeground(dpy, gc, black);
	XDrawString(dpy, field, gc, FIELD_WIDTH/2-width/2+1, FIELD_HEIGHT/2+1, msg, strlen(msg));
	XSetForeground(dpy, gc, white);
	XDrawString(dpy, field, gc, FIELD_WIDTH/2-width/2, FIELD_HEIGHT/2, msg, strlen(msg));
	XFlush(dpy);
}

static void
display_replay()
{
	static int toggle = 0;
	char msg[256];

	if (game.state == S_NEW)
	{
		XClearWindow(dpy,panel);
		XFlush(dpy);
		return;
	}
	sprintf(msg,"Replay");
	XSetFont(dpy, gc, font[FONT_TITLE]->fid);
	if (toggle)
	{
		XSetForeground(dpy, gc, white);
	}
	else
	{
		XSetForeground(dpy, gc, black);
	}
	toggle = !toggle;
	XDrawString(dpy, panel, gc, FIELD_WIDTH/2, PANEL_HEIGHT/2, msg, strlen(msg));
	XFlush(dpy);
}

static void
display_ready_flags()
{
#define BULB_WIDTH 40
	long color;
	long x1,y1,x2,y2;

	if (game.ready[0])
	{
		color = get_color(1);
	}
	else
	{
		color = get_color(0);
	}
	x1 = proj_x(GAME_DEPTH/4,3*GAME_WIDTH/4-BULB_WIDTH/2,BULB_WIDTH);
	y1 = proj_y(GAME_DEPTH/4,3*GAME_WIDTH/4-BULB_WIDTH/2,BULB_WIDTH);
	x2 = proj_x(GAME_DEPTH/4,3*GAME_WIDTH/4+BULB_WIDTH/2,0);
	y2 = proj_y(GAME_DEPTH/4,3*GAME_WIDTH/4+BULB_WIDTH/2,0);
	XSetForeground(dpy,gc,color);
	XFillArc(dpy,field,gc,x1,y1,x2-x1,y2-y1,0,180*64);
	XSetForeground(dpy,gc,white);
	XDrawArc(dpy,field,gc,x1,y1,x2-x1,y2-y1,0,180*64);
	if (game.ready[1])
	{
		color = get_color(1);
	}
	else
	{
		color = get_color(0);
	}
	x1 = proj_x(3*GAME_DEPTH/4,-3*GAME_WIDTH/4-BULB_WIDTH/2,BULB_WIDTH);
	y1 = proj_y(3*GAME_DEPTH/4,-3*GAME_WIDTH/4-BULB_WIDTH/2,BULB_WIDTH);
	x2 = proj_x(3*GAME_DEPTH/4,-3*GAME_WIDTH/4+BULB_WIDTH/2,0);
	y2 = proj_y(3*GAME_DEPTH/4,-3*GAME_WIDTH/4+BULB_WIDTH/2,0);
	XSetForeground(dpy,gc,color);
	XFillArc(dpy,field,gc,x1,y1,x2-x1,y2-y1,0,180*64);
	XSetForeground(dpy,gc,white);
	XDrawArc(dpy,field,gc,x1,y1,x2-x1,y2-y1,0,180*64);

	XFlush(dpy);
}

static void
display_intro()
{
	int		width;

	XClearWindow(dpy, field);
	XSetFont(dpy, gc, font[FONT_INTRO]->fid);
	width = XTextWidth(font[FONT_INTRO],"Xpuck",strlen("Xpuck"));
	XSetForeground(dpy, gc, get_color(4));
	XDrawString(dpy, field, gc, FIELD_WIDTH/2-width/2+2, FIELD_HEIGHT/2+2, "Xpuck", strlen("Xpuck"));
	XSetForeground(dpy, gc, get_color(0));
	XDrawString(dpy, field, gc, FIELD_WIDTH/2-width/2, FIELD_HEIGHT/2, "Xpuck", strlen("Xpuck"));
	XFlush(dpy);

	XSetFont(dpy, gc, font[FONT_SCORE]->fid);
	XSetForeground(dpy, gc, get_color(5));
	XDrawString(dpy, field, gc, 20, FIELD_HEIGHT-20, "O.Debon (1997)", strlen("O.Debon (1997)"));
	XFlush(dpy);
}
