/**********************************************************/
/**                                                       */
/**   %Z% %M% Version %I% (%G%)                           */
/**                                                       */
/**********************************************************/
/**                                                       */
/**   AUTEUR :  O.DEBON                                   */
/**                                                       */
/**   DATE DE CREATION :  23/12/96                        */
/**                                                       */
/**********************************************************/
/**                                                       */
/**   MODIFICATIONS :                                     */
/**                                                       */
/**   Date         Auteur           Commentaires          */
/**                                                       */
/**                                                       */
/**********************************************************/
#ifndef _PUCK_H_
#define _PUCK_H_

#define GAME_VERSION		"1.0"

/* Scheduling Time */
#define TICK			20000

/* Font Definitions */
#define NB_FONTS		5
#define FONT_TITLE		0
#define FONT_TEXT		1
#define FONT_DIGIT		2
#define FONT_SCORE		3
#define FONT_INTRO		4

/* Window definition */
#define FIELD_WIDTH		800
#define FIELD_HEIGHT		400
#define PANEL_HEIGHT		60
#define BORDER_WIDTH		2

/* Game visual parameters */
#define HORIZON			400
#define GAME_DEPTH		1600
#define GAME_WIDTH		800
#define GAME_BORDER		40
#define GAME_PLAYER_ZONE	300
#define FOCALE			400
#define GAME_PORT		7000

/* Set and Game definition */
#define POINT_MAX		15
#define NB_SETS			5

/* Racket definition */
#define RACKET_WIDTH		200
#define RACKET_HEIGHT		60
#define RACKET_DX_FACT		2.2
#define RACKET_DY_FACT		2.5

/* Puck definition */
#define PUCK_WIDTH		70
#define PUCK_HEIGHT		30
#define PUCK_DX_FACT		1.2
#define PUCK_DY_FACT		1.2
#define PUCK_DXY_MAX		80

/* Structures */

/* Puck Handle */
typedef
struct s_puck {
	float	x,y;	/* Puck position */
	float	last_x,last_y;	/* Last puck position */
	float	last_X,last_Y;	/* Last drawn puck position */
	float	dx,dy;	/* Puck delta move (speed actually) */
} Puck;

/* Racket Handle */
typedef
struct s_racket {
	float	x,y;	/* Racket position */
	float	last_x,last_y;	/* Last Racket position */
	float	last_X,last_Y;	/* Last drawn racket position */
	float	dx,dy;	/* Racket velocity */
} Racket;

/* Game states */
typedef
enum {
	S_START,
	S_PLAY,
	S_NEW,
	S_LOST,
	S_GAME_OVER,
	S_REPLAY,
	S_INTRO,
	S_QUIT
} State;

#include <time.h>

typedef
enum {
	GAME_MASTER,
	GAME_SLAVE,
	GAME_AUTO
} Mode;

#define MAX_NAME 10
#define MAX_PATH 256

/* Global Game Data */
typedef
struct s_game {
	State	state;
	Mode 	mode;	/* Master, Slave, Auto */
	int	server;	/* File descriptor for incoming messages */
	int	client; /* File descriptor for outgoing messages */
	char *  server_name;	/* Name of Master host */
	char    hostname[MAX_NAME];	/* Name of local machine */
	int	port;			/* Socket port */
	int	score[NB_SETS][2];	/* Scores, 0 is local, 1 is remote player */
	int	set;			/* Current set number */
	int	ready[2];		/* Ready flags, 0 is local, 1 is remote */
	char    my_name[MAX_NAME];	/* Name of local player */
	char    opponent_name[MAX_NAME]; /* Name of remote player */
	int	to_serve;		/* Who is to serve next, 0 is local, 1 is remote */
	char    xpm_file[MAX_PATH];	/* Xpm file path name, given to Remote */
	char    opponent_xpm_file[MAX_PATH]; /* Xpm file path name, received from Remote */
	void    (*init_racket2)();	/* Automatic Mode, Init */
	void    (*move_racket2)();	/* Automatic Mode, Move */
} Game;

/* Connection structure */
typedef
struct s_connection {
	char player_name[MAX_NAME];	/* Name of remote */
	char xpm_file[MAX_PATH];	/* Xpm file name of remote */
	int port;			/* Socket port number */
} Connection;

/* Declaration of Projection Functions */
long proj_x(float x, float y, float z);
long proj_y(float x, float y, float z);
/* Declaration of get_color */
long get_color(int);

/* MACROS */

/* Return a couple */
#define XY(x,y,z) proj_x(x,y,z),proj_y(x,y,z)

/* Fill in a XPoint entry */
#define XPOINT(P,X,Y,Z) \
	P.x = proj_x(X,Y,Z), P.y = proj_y(X,Y,Z)

#endif /* _PUCK_H_ */
